import RiiiverSDK

@objc(TriggerPieceCoreSample)
class TriggerPieceCoreSample: NSObject, ERTriggerBlockExecutor {
    weak var delegate: ERTriggerBlockDelegate?
    var identifier: String?
    
    // TriggerPieceCoreSample の実行要求通知がきたら実行される
    // called when the notification of the TriggerPieceCoreSample execution comes.
    @objc private func didReceiveTriggerPieceCoreSampleTrigger(_ notification: Notification) {
        guard let userInfo = notification.userInfo else { return }
        
        // Check the ID - IDのチェック
        guard let identifier = userInfo[TriggerNotification.Key.Identifier] as? String else {
            print("TriggerPieceCoreSample error in execution. reason:no ID")
            return
        }
        if identifier != self.identifier {
            // does not fire because the ID is wrong. - IDが異なるため発火しない
            return
        }
        
        // TriggerPieceCoreSampleのアウトプット - Get the text field's string and set it as a output of the T piece.
        var output = [ "triggerSampleInfo" : "" ]
        output["triggerSampleInfo"] = userInfo[TriggerNotification.Key.Output] as? String
        print("TriggerPieceCoreSample Trigger is fired! output:\(String(describing: output))")
        
        // Trigger with outpur information - output情報を設定してTrigger発火
        delegate!.didTrigger(outputJson: output)
    }
    
    func createTrigger(blockJsonString: String, preferenceJson: Dictionary<String, Any>, applet: ERApplet) -> Bool {
        print("TriggerPieceCoreSample:\(self.identifier ?? "")]")
        identifier = applet.getId()
        return true
    }
    
    func enable(delegate: ERTriggerBlockDelegate) -> Bool {
        print("TriggerPieceCoreSample enable")
        self.delegate = delegate
        
        // sampleTrigger実行通知を監視を開始する。通知を受けたら didReceiveTriggerPieceCoreSampleTrigger() を実行する
        // Starts ovserving the notification for the "sampleTrigger" execution.  When received the notification, executes didReceiveTriggerPieceCoreSampleTrigger().
        NotificationCenter.default.addObserver(self, selector: #selector(didReceiveTriggerPieceCoreSampleTrigger(_:)), name: TriggerNotification.Name.sampleTrigger, object: nil)
        return true
    }
    
    func disable() {
        print("TriggerPieceCoreSample disable")
        delegate = nil
        
        // sampleTrigger実行通知を監視を終了する。
        // Stop observing the notification for the sampleTrigger execution.
        NotificationCenter.default.removeObserver(self, name: TriggerNotification.Name.sampleTrigger, object: nil)
    }
}
