import RiiiverSDK

@objc(TriggerPieceCoreSample2)
class TriggerPieceCoreSample2: NSObject, ERTriggerBlockExecutor {
    weak var delegate: ERTriggerBlockDelegate?
    var identifier: String?

    // TriggerPieceCoreSample2 の実行要求通知がきたら実行される
    // called when the notification of TriggerPieceCoreSample2 execution comes.
    @objc private func didReceiveTriggerPieceCoreSample2Trigger(_ notification: Notification) {
        guard let userInfo = notification.userInfo else { return }
        
        // Check ID - IDのチェック
        guard let identifier = userInfo[TriggerNotification.Key.Identifier] as? String else {
            print("TriggerPieceCoreSample2 error in execution. reason:no ID")
            return
        }
        if identifier != self.identifier {
            // does not fire because ID is wrong - IDが異なるため発火しない
            return
        }

        // TriggerPieceCoreSampleのアウトプット - Get the text field's string and set it as a output of the T piece.
        var output = [ "triggerSampleInfo" : "" ]
        output["triggerSampleInfo"] = userInfo[TriggerNotification.Key.Output] as? String
        print("TriggerPieceCoreSample2 Trigger is fired! output:\(String(describing: output))")

        // output情報を設定してTrigger発火
        // Trigger with output data
        delegate!.didTrigger(outputJson: output)
    }
    
    func createTrigger(blockJsonString: String, preferenceJson: Dictionary<String, Any>, applet: ERApplet) -> Bool {
        print("TriggerPieceCoreSample2:\(self.identifier ?? "")]")
        identifier = applet.getId()
        return true
    }
    
    func enable(delegate: ERTriggerBlockDelegate) -> Bool {
        print("TriggerPieceCoreSample2 enable")
        self.delegate = delegate

        // sampleTrigger2実行通知を監視を開始する。通知を受けたら didReceiveTriggerPieceCoreSample2Trigger() を実行する
        // Starts ovserving the notification for the "sampleTrigger2" execution.  When received the notification, executes didReceiveTriggerPieceCoreSample2Trigger().
        NotificationCenter.default.addObserver(self, selector: #selector(didReceiveTriggerPieceCoreSample2Trigger(_:)), name: TriggerNotification.Name.sampleTrigger2, object: nil)
        return true
    }
    
    func disable() {
        print("TriggerPieceCoreSample2 disable")
        delegate = nil

        // sampleTrigger2実行通知を監視を終了する。
        // Stop observing the notification for the sampleTrigger2 execution.
        NotificationCenter.default.removeObserver(self, name: TriggerNotification.Name.sampleTrigger2, object: nil)
    }
}
