//
//  UIImageView.swift
//  sample
//
//  Created by CITIZEN Watch Co.,Ltd on 2019/10/28.
//  Copyright © 2019 CITIZEN Watch Co.,Ltd. All rights reserved.
//

import Foundation
import UIKit

extension UIImageView {
    
    static let imageCache = NSCache<AnyObject, AnyObject>()
    
    func setImage(fromUrl: String) {
        
        // If URL have been cached alrady, we use cached one.  -  既にキャッシュされているURLの場合はそちらを使用
        if let cacheImage = UIImageView.imageCache.object(forKey: fromUrl as AnyObject) as? UIImage {
            self.image = cacheImage
            return
        }
        //　Create the indicator to display "Image loading...".  -  画像を読み込み中を示すインジケータを生成
        var ActivityIndicator: UIActivityIndicatorView!
        ActivityIndicator = UIActivityIndicatorView()
        ActivityIndicator.frame = CGRect(x: 0, y: 0, width: 100, height: 100)
        ActivityIndicator.center = self.center
        ActivityIndicator.hidesWhenStopped = true
        ActivityIndicator.style = .white
        ActivityIndicator.backgroundColor = .gray
        
        if let imageUrl = URL(string: fromUrl){
            //　Start animation for image loading indicator.  -  画像を読み込み中を示すインジケータを表示開始
            ActivityIndicator.startAnimating()
            
            DispatchQueue.global().async {
                var image:UIImage? = nil;
                do {
                    let data = try Data(contentsOf: imageUrl)
                    image = UIImage(data: data)
                    // Cache the image  -  画像をキャッシュ
                    UIImageView.imageCache.setObject(image!, forKey: fromUrl as AnyObject)
                } catch let error {
                    print("UIImageView failed. reason: \(error)")
                }
                DispatchQueue.main.async {
                    self.image = image
                    //　Stop the amination for "image loading."  -  画像を読み込み中を示すインジケータを表示終了
                    ActivityIndicator.stopAnimating()
                }
            }
        }
        self.addSubview(ActivityIndicator)
    }
}
